@@ -172,8 +172,8 @@ pub fn archive_dyn(
                             where
                                 Archived<#ty>: for<'a> Deserialize<#ty, (dyn DynDeserializer + 'a)>,
                             {
-                                unsafe fn deserialize_dyn(&self, deserializer: &mut dyn DynDeserializer) -> Result<*mut (), DynError> {
-                                    let result = deserializer.alloc_dyn(core::alloc::Layout::new::<#ty>())? as *mut #ty;
+                                unsafe fn deserialize_dyn(&self, deserializer: &mut dyn DynDeserializer, alloc: &mut dyn FnMut(Layout) -> *mut u8) -> Result<*mut (), DynError> {
+                                    let result = alloc(core::alloc::Layout::new::<#ty>()).cast::<#ty>();
                                     result.write(self.deserialize(deserializer)?);
                                     Ok(result as *mut ())
                                 }
@@ -196,6 +196,7 @@ pub fn archive_dyn(
                     #input
 
                     const _: () = {
+                        use core::alloc::Layout;
                         use rkyv::{
                             Archived,
                             Deserialize,
@@ -264,9 +265,9 @@ pub fn archive_dyn(
                         quote! {
                             impl<__T: #name<#generic_args> + DeserializeDyn<dyn #serialize_trait<#generic_args>>, #generic_params> #deserialize_trait<#generic_args> for __T {}
 
-                            impl<__D: Deserializer + ?Sized, #generic_params> DeserializeUnsized<dyn #serialize_trait<#generic_args>, __D> for dyn #deserialize_trait<#generic_args> {
-                                unsafe fn deserialize_unsized(&self, mut deserializer: &mut __D) -> Result<*mut (), __D::Error> {
-                                    self.deserialize_dyn(&mut deserializer).map_err(|e| *e.downcast().unwrap())
+                            impl<__D: Fallible + ?Sized, #generic_params> DeserializeUnsized<dyn #serialize_trait<#generic_args>, __D> for dyn #deserialize_trait<#generic_args> {
+                                unsafe fn deserialize_unsized(&self, mut deserializer: &mut __D, mut alloc: impl FnMut(Layout) -> *mut u8) -> Result<*mut (), __D::Error> {
+                                    self.deserialize_dyn(&mut deserializer, &mut alloc).map_err(|e| *e.downcast().unwrap())
                                 }
 
                                 fn deserialize_metadata(&self, mut deserializer: &mut __D) -> Result<<dyn #serialize_trait<#generic_args> as ptr_meta::Pointee>::Metadata, __D::Error> {
@@ -305,11 +306,20 @@ pub fn archive_dyn(
             #[cfg(feature = "validation")]
             let validation_impl = quote! {
                 use bytecheck::CheckBytes;
+                use rkyv::validation::LayoutRaw;
                 use rkyv_dyn::validation::{CHECK_BYTES_REGISTRY, CheckDynError, DynContext};
 
+                impl<#generic_params> LayoutRaw for (dyn #deserialize_trait<#generic_args> + '_) {
+                    fn layout_raw(value: *const Self) -> Layout {
+                        let metadata = ptr_meta::metadata(value);
+                        metadata.layout()
+                    }
+                }
+
                 impl<#generic_params> CheckBytes<dyn DynContext + '_> for (dyn #deserialize_trait<#generic_args> + '_) {
                     type Error = CheckDynError;
 
+                    #[inline]
                     unsafe fn check_bytes<'a>(value: *const Self, context: &mut (dyn DynContext + '_)) -> Result<&'a Self, Self::Error> {
                         let vtable = core::mem::transmute(ptr_meta::metadata(value));
                         if let Some(validation) = CHECK_BYTES_REGISTRY.get(vtable) {
@@ -324,6 +334,7 @@ pub fn archive_dyn(
                 impl<__C: DynContext, #generic_params> CheckBytes<__C> for (dyn #deserialize_trait<#generic_args> + '_) {
                     type Error = CheckDynError;
 
+                    #[inline]
                     unsafe fn check_bytes<'a>(value: *const Self, context: &mut __C) -> Result<&'a Self, Self::Error> {
                         Self::check_bytes(value, context as &mut dyn DynContext)
                     }
@@ -343,15 +354,16 @@ pub fn archive_dyn(
                 #deserialize_trait_def
 
                 const _: ()  = {
-                    use core::mem::MaybeUninit;
+                    use core::alloc::Layout;
                     use rkyv::{
-                        de::Deserializer,
-                        ser::Serializer,
+                        ser::{ScratchSpace, Serializer},
+                        Archive,
                         Archived,
                         ArchivedMetadata,
                         ArchivePointee,
                         ArchiveUnsized,
                         DeserializeUnsized,
+                        Fallible,
                         SerializeUnsized,
                     };
                     use rkyv_dyn::{
@@ -384,7 +396,7 @@ pub fn archive_dyn(
                         type Archived = dyn #deserialize_trait<#generic_args>;
                         type MetadataResolver = ();
 
-                        fn resolve_metadata(&self, _: usize, _: Self::MetadataResolver, out: &mut MaybeUninit<ArchivedMetadata<Self>>) {
+                        unsafe fn resolve_metadata(&self, _: usize, _: Self::MetadataResolver, out: *mut ArchivedMetadata<Self>) {
                             ArchivedDynMetadata::emplace(self.archived_type_id(), out);
                         }
                     }
@@ -397,7 +409,7 @@ pub fn archive_dyn(
                         }
                     }
 
-                    impl<__S: Serializer + ?Sized, #generic_params> SerializeUnsized<__S> for dyn #serialize_trait<#generic_args> {
+                    impl<__S: ScratchSpace + Serializer + ?Sized, #generic_params> SerializeUnsized<__S> for dyn #serialize_trait<#generic_args> {
                         fn serialize_unsized(&self, mut serializer: &mut __S) -> Result<usize, __S::Error> {
                             self.serialize_dyn(&mut serializer).map_err(|e| *e.downcast::<__S::Error>().unwrap())
                         }
